package com.brilliance.isc.transfer.transaction;

import com.brilliance.isc.bo.lg.NitengStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.NitengStoreVo;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;


/**
 * Gitame Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface NitengStructMapper extends IBaseStructMapper<NitengStoreBo, NitengStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    NitengStructMapper INSTANCE = Mappers.getMapper(NitengStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    NitengStoreBo voTransferToBo(NitengStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
            @Mapping(target = "oldnidgrp",ignore = true),
            @Mapping(target = "nidgrp",ignore = true),
    })
    void voUpdateToBo(NitengStoreVo vo, @MappingTarget NitengStoreBo bo);

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    NitengStoreVo boTransferToVo(NitengStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(NitengStoreBo bo, @MappingTarget NitengStoreVo vo);
}
